﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO.Ports;
using System.IO;
using System.Threading;

namespace Uart_PC_Bootloader
{
    public partial class Form1 : Form
    {
        
        public Form1()
        {
            InitializeComponent();
        }

        #region Globale Variablen

        SerialPort Schnittstelle = new SerialPort();

        #endregion

        #region Form1

        private void Form1_Load(object sender, EventArgs e)
        {
            cb_ComPorts.Items.AddRange(SerialPort.GetPortNames());

            //Port einstellen, Fehlerabsicherung (Portnummer)
            if (cb_ComPorts.Items.Count > Properties.Settings.Default.stan_cb_Port)
                cb_ComPorts.SelectedIndex = Properties.Settings.Default.stan_cb_Port;
            else cb_ComPorts.SelectedIndex = 0;

            cb_BaudRate.SelectedIndex = Properties.Settings.Default.stan_cb_Baud;

            btn_StoppSeriell.Enabled = false;
        }

        private void Form1_FormClosing(object sender, FormClosingEventArgs e)
        {
            if (Schnittstelle.IsOpen) Schnittstelle.Close();
            Properties.Settings.Default.Save();
        }

        #endregion

        #region Thread

        void Nebenthread(object sender, SerialDataReceivedEventArgs e) //Diese Methode wird ausgelöst. Wurde vorher abonniert
        {
            
            try
            {
                string serialIn = Schnittstelle.ReadLine();
                tb_SeriellOut.Invoke(new EventHandler(delegate { tb_SeriellOut.AppendText(serialIn + "\r\n"); }));

                //if (serialIn.IndexOf("Int:") != -1)
            }
            catch
            {

            }
        }

        #endregion

        #region Methoden

        // String an serielle Schnitstelle ausgeben ( Write), Fehlerabsicherung (Schnitstelle geöffnet)
        private void seriellWriteLine(string text)
        {
            try
            {
                if (Schnittstelle.IsOpen)
                {
                    Schnittstelle.Write(text);
                    //Schnittstelle.WriteLine(text);
                }
                else
                    if (MessageBox.Show("Schnittstelle ist nicht verbunden, soll sie nun gestartet werden ? ", "Serielle Schnittstelle starten!", MessageBoxButtons.YesNoCancel) == DialogResult.Yes)
                    {
                        seriellInI();
                    }
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message.ToString(), "Fehler in serieller Schnittstelle");
            }

        }

        // Schnitstelle initialisieren und Starten, Fehlerabsicherung (Erfolg )
        private void seriellInI()
        {
            Schnittstelle.PortName = cb_ComPorts.SelectedItem.ToString();
            Schnittstelle.BaudRate = int.Parse(cb_BaudRate.SelectedItem.ToString());
            //Schnittstelle.PortName = "COM4";
            //Schnittstelle.BaudRate = 9600;
            Schnittstelle.Parity = Parity.None;
            Schnittstelle.DataBits = 8;
            Schnittstelle.StopBits = StopBits.One;
            Schnittstelle.Handshake = Handshake.RequestToSendXOnXOff;
            Schnittstelle.ReadTimeout = 500;
            Schnittstelle.WriteTimeout = 500;
            try
            {
                // Schummeln ! Softwareflußsteuerung funktioniert nur nach umkonfigurierung von RequestToSendXOnXOff zu XOnXOff
                // mal nachgoogeln
                // wäre sinvoll
                Schnittstelle.Open(); //Serialport öffnen
                Schnittstelle.Close();
                Schnittstelle.Handshake = Handshake.XOnXOff;
                Schnittstelle.Open();

                if (Schnittstelle.IsOpen)
                {
                    btn_StartSeriell.Enabled = false; //Connect_Button blass
                    btn_StoppSeriell.Enabled = true;
                    cb_BaudRate.Enabled = false;
                    cb_ComPorts.Enabled = false;
                }
            }
            catch
            {
                MessageBox.Show("Keine Verbindung möglich", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            Schnittstelle.DataReceived += new SerialDataReceivedEventHandler(Nebenthread); // Abo_Event abonnieren
        }

        //Hexfile auslesen, Bytegröße berechnen, Fehlerabsicherung (MsgBox Datei zu groß)
        private byte[] ReadConvertHexFile(string path) //static
        {
            string complete = string.Empty;
            string hilfsstring = string.Empty;
            int i = 0;
            int bytes = 0;
            using (StreamReader sr = new StreamReader(path, Encoding.Default))
            {
                while (sr.Peek() != -1)
                {
                    hilfsstring = sr.ReadLine();
                    complete += hilfsstring;
                    i++;
                    tb_SeriellOut.AppendText(i.ToString() + ". Zeile gelesen :  " + hilfsstring + "\r\n");
                    // auswertung  hexfile größe 
                    bytes += Int32.Parse(hilfsstring.Substring(1, 2), System.Globalization.NumberStyles.HexNumber);
                }
            }
            tb_SeriellOut.AppendText("Anzahl Bytes :  " + bytes.ToString() + "\r\n");
            if (bytes > 6144)
            {
                DialogResult result = MessageBox.Show("Warnung : Flashdatei ist größer als 6144 bytes !\r\n Flash abbrechen ?", "Warung", MessageBoxButtons.YesNo);
                if (result == System.Windows.Forms.DialogResult.Yes)
                    complete = string.Empty;
                // abbruch oder weiter 
            }
            return Encoding.ASCII.GetBytes(complete);
        }

        #endregion
        
        #region Events

        // Hexdatei FileopenDialog, Datei laden, Senden, Fehlerabsicherung ( diverses)
        private void btn_sendfile_Click(object sender, EventArgs e)
        {
            string pfad = string.Empty;

            OpenFileDialog openFileDialog1 = new OpenFileDialog();
            openFileDialog1.Filter = "Hex Files (*.hex)|*.hex|All files (*.*)|*.*";
            if (openFileDialog1.ShowDialog() == DialogResult.OK)
                pfad = openFileDialog1.FileName;

            if (File.Exists(pfad))
            {
                tb_SeriellOut.AppendText("Lese HexDatei : " + pfad + "\r\n");
                byte[] array = ReadConvertHexFile(pfad);
                if (array.Length > 0)
                {
                    DialogResult result = MessageBox.Show("Flashvorgang starten ? ", "Flasch", MessageBoxButtons.YesNo, MessageBoxIcon.Warning);
                    if (result == System.Windows.Forms.DialogResult.Yes)
                    {
                        tb_SeriellOut.AppendText("Sende Daten bitte warten..." + "\r\n");
                        for (int i = 0; i < array.Length; i++)
                        {
                            try
                            {
                                if (Schnittstelle.IsOpen)
                                {
                                    if (array[i] == ':') tb_SeriellOut.AppendText("#");
                                    Schnittstelle.Write(array, i, 1);
                                }
                                else
                                    if (MessageBox.Show("Schnittstelle ist nicht verbunden, soll sie nun gestartet werden ? ", "Serielle Schnittstelle starten!", MessageBoxButtons.YesNoCancel) == DialogResult.Yes)
                                    {
                                        seriellInI();
                                    }
                            }
                            catch (Exception ex)
                            {
                                MessageBox.Show(ex.Message.ToString(), "Fehler in serieller Schnittstelle");
                            }
                        }
                        tb_SeriellOut.AppendText("\r\n" + "Daten gesendet, bitte warten" + "\r\n");
                    }
                }
            }
            else
                MessageBox.Show("Datei existiert nicht ! Abbruch des Flash Vorganges");
        }

        // Schnitstelle starten
        private void btn_StartSeriell_Click_1(object sender, EventArgs e)
        {
            seriellInI(); 
            btn_StartSeriell.Enabled = false;
        }

        // Schnitstelle stoppen
        private void btn_StoppSeriell_Click(object sender, EventArgs e)
        {
            Schnittstelle.Close();
            btn_StartSeriell.Enabled = true;
            btn_StoppSeriell.Enabled = false;
            cb_BaudRate.Enabled = true;
            cb_ComPorts.Enabled = true;
        }

        // String an Schnitstelle senden
        private void btn_Send_Click(object sender, EventArgs e)
        {
             seriellWriteLine(tb_TextToSend.Text);
             tb_TextToSend.Clear();
    }

        // String an Schnitstelle senden
        private void tb_TextToSend_KeyUp(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == Keys.Return)
            {
                seriellWriteLine(tb_TextToSend.Text);
                tb_TextToSend.Clear();
            }
        }

        private void cb_ComPorts_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (cb_ComPorts.SelectedIndex != Properties.Settings.Default.stan_cb_Port)
                btn_standardPort.Enabled = true;
            else
                btn_standardPort.Enabled = false;
        }

        private void cb_ComPorts_Click(object sender, EventArgs e)
        {
            cb_ComPorts.Items.Clear();
            cb_ComPorts.Items.AddRange(SerialPort.GetPortNames());
        }

        private void cb_BaudRate_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (cb_BaudRate.SelectedIndex != Properties.Settings.Default.stan_cb_Baud)
                btn_standardBaud.Enabled = true;
            else
                btn_standardBaud.Enabled = false;
        }

        private void btn_standardPort_Click(object sender, EventArgs e)
        {
            Properties.Settings.Default.stan_cb_Port = cb_ComPorts.SelectedIndex;
            btn_standardPort.Enabled = false;
        }

        private void btn_standardBaud_Click_Click(object sender, EventArgs e)
        {
            Properties.Settings.Default.stan_cb_Baud = cb_BaudRate.SelectedIndex;
            btn_standardBaud.Enabled = false;
        }

        #endregion
    }
}